@@ -1,18 +1,21 @@
 #!/usr/bin/env python
 
 import argparse
+from datetime import datetime, timedelta
 import getpass
 import hashlib
 import logging
 import os
 import requests
 import shutil
+import socket
 from string import Template
 import subprocess
 import sys
 import time
 
 import edgebox
+from edgebox import github
 from edgebox.master_controller import MC, prompt, RESERVED_CLOUDLET_ORGS
 
 logging.basicConfig(
@@ -168,10 +171,46 @@ def load_deps(mc, args, version):
     os.environ["PATH"] = "{0}:{1}".format(deps_dir, os.environ["PATH"])
     os.environ["GOPATH"] = deps_dir
 
+def create_cloudlet_pool(mc):
+    pool = mc.get_cloudlet_pool()
+    if not pool:
+        mc.banner("Creating cloudlet pool")
+        mc.create_cloudlet_pool()
+
+def delete_cloudlet_pool(mc):
+    pool = mc.get_cloudlet_pool()
+    if pool:
+        pool = pool[0]
+        if "cloudlets" not in pool or len(pool["cloudlets"]) < 1:
+            mc.banner("Deleting cloudlet pool")
+            mc.delete_cloudlet_pool()
+
 def create_cloudlet(mc):
     mc.banner("Creating cloudlet")
     mc.create_cloudlet()
 
+def add_cloudlet_to_pool(mc):
+    mc.banner("Adding cloudlet to pool")
+    try:
+        mc.add_cloudlet_to_pool()
+    except Exception as e:
+        if "Cloudlet already part of pool" in str(e):
+            print("Cloudlet already part of pool")
+        else:
+            raise e
+
+def add_org_to_cloudlet_pool(mc, org):
+    if not mc.show_cloudlet_pool_orgs(org=org):
+        mc.banner("Adding developer org to cloudlet pool")
+        mc.link_org_to_pool(org)
+        print()
+
+def remove_orgs_from_cloudlet_pool(mc):
+    for orgdet in mc.show_cloudlet_pool_orgs():
+        org = orgdet["Org"]
+        mc.banner("Removing developer org from cloudlet pool: {}".format(org))
+        mc.unlink_org_from_pool(org)
+
 def generate_access_key(mc, access_key_file):
     mc.banner("Generating access key")
     accesskey = mc.get_access_key()
@@ -179,6 +218,20 @@ def generate_access_key(mc, access_key_file):
         f.write(accesskey)
     return access_key_file
 
+def get_notify_srv_port():
+    for port in range(51001, 51100):
+        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
+            if s.connect_ex(("127.0.0.1", port)) != 0:
+                logging.debug("Port free: {}".format(port))
+                break
+
+                logging.debug("Port in use: {}".format(port))
+                continue
+    else:
+        raise Exception("Unable to find free port for notify server")
+
+    return str(port)
+
 def start_crm(mc, access_key_file):
     mc.banner("Starting CRM")
 
@@ -195,8 +248,12 @@ def start_crm(mc, access_key_file):
             mc.deploy_env)
     os.environ["JAEGER_ENDPOINT"] = jaeger
 
+    srv_port = get_notify_srv_port()
+    logging.debug("Notify server port: {}".format(srv_port))
+
     cmd = [ "crmserver",
 	    "--notifyAddrs", mc.controller + ":37001",
+            "--notifySrvAddr", "127.0.0.1:" + srv_port,
 	    "--cloudletKey",
             '{{"organization": "{0}","name":"{1}"}}'.format(mc.cloudlet_org, mc.cloudlet),
 	    "--hostname", "crm1",
@@ -242,7 +299,14 @@ def cleanup_cloudlet(mc):
             # Cloudlet has already been deleted
             print("Cloudlet does not exist")
         else:
-            raise e
+            # Wait for cloudlet to be deleted
+            start = datetime.now()
+            while mc.get_cloudlet():
+                logging.debug("Cloudlet still present; waiting...")
+                if datetime.now() - start > timedelta(seconds=30):
+                    # Failed to delete cloudlet
+                    raise e
+                time.sleep(2)
 
 def cleanup_crm(mc, access_key_file):
     mc.banner("Killing CRM process")
@@ -279,7 +343,10 @@ def cleanup_docker(mc):
 
 def is_cloudlet_ready(mc):
     info = mc.get_cloudlet_info()
-    return info[0]["state"] == 5
+    if info:
+        logging.debug("Cloudlet status: {}".format(info))
+        return info[0]["state"] == 5
+    return False
 
 def edgebox_create(args):
     config, access_key_file = init(args)
@@ -289,14 +356,19 @@ def edgebox_create(args):
     mc.save()
 
     load_deps(mc, args, edgebox.__version__)
+    if not mc.docker_logged_in:
+        print("\nNot logged in to docker")
+        print("  docker login -u {} {}".format(mc.username, mc.docker))
+        sys.exit(2)
 
     print("\nCreating edgebox cloudlet:")
     print(mc)
     if not mc.confirm_continue():
-        os.remove(config)
         sys.exit("Not creating edgebox")
 
+    create_cloudlet_pool(mc)
     create_cloudlet(mc)
+    add_cloudlet_to_pool(mc)
     generate_access_key(mc, access_key_file)
     start_crm(mc, access_key_file)
 
@@ -320,12 +392,18 @@ def edgebox_delete(args):
 
     mc = MC(args.name, config)
 
+    # Load user credentials
+    mc.username
+    mc.password
+
     print("\nDeleting edgebox cloudlet:")
     print(mc)
     if not mc.confirm_continue():
         sys.exit("Not deleting cloudlet")
 
     cleanup_cloudlet(mc)
+    remove_orgs_from_cloudlet_pool(mc)
+    delete_cloudlet_pool(mc)
     cleanup_docker(mc)
     cleanup_crm(mc, access_key_file)
 
@@ -346,15 +424,22 @@ def edgebox_show(args):
     print(mc)
 
     cloudlet = mc.get_cloudlet()
+    if not cloudlet:
+        print("    Cloudlet not present")
+        return
+
     cloudlet_state = cloudlet[0]["state"]
     if cloudlet_state != 5:
         print("    Tracked State: {}".format(CLOUDLET_STATES.get(
             cloudlet_state, cloudlet_state)))
 
     info = mc.get_cloudlet_info()
-    cloudletinfo_state = info[0]["state"]
-    print("    Cloudlet State: {}".format(CLOUDLETINFO_STATES.get(
-        cloudletinfo_state, cloudletinfo_state)))
+    try:
+        cloudletinfo_state = info[0]["state"]
+        print("    Cloudlet State: {}".format(CLOUDLETINFO_STATES.get(
+            cloudletinfo_state, cloudletinfo_state)))
+    except IndexError:
+        print("    Cloudlet not ready")
 
 def edgebox_github(args):
     config, _ = init(args)
@@ -372,6 +457,7 @@ def edgebox_github(args):
         sys.exit("Not part of any developer orgs")
 
     org = prompt("Pick an org", choices=orgs, default=orgs[0])
+    add_org_to_cloudlet_pool(mc, org)
 
     repo_conf_dir = os.path.join(args.repo, ".mobiledgex")
     if not os.path.exists(repo_conf_dir):
@@ -384,77 +470,34 @@ def edgebox_github(args):
     # Assume Github repo name is the app name
     app_name = os.path.basename(os.path.realpath(args.repo))
 
-    # Template app config
-    app_tmpl_file = os.path.join(os.path.dirname(__file__), "app.yml.tmpl")
-    with open(app_tmpl_file) as f:
-        app_tmpl = f.read()
-
-    docker = "docker.mobiledgex.net" if mc.deploy_env == "main" \
-        else "docker-{}.mobiledgex.net".format(mc.deploy_env)
-
-    t = Template(app_tmpl)
-    app_yml = t.substitute({
+    data = {
         "appname": app_name,
         "devorg": org,
-        "devorg_lc": org.lower(),
-        "docker": docker,
+        "docker": mc.docker,
         "region": mc.region,
-    })
-
-    app_yml_file = os.path.join(repo_conf_dir, "app.yml")
-    if os.path.exists(app_yml_file):
-        nfile = app_yml_file + ".new"
-        print("File exists: {0}. Writing to {1}".format(
-            app_yml_file, nfile))
-        app_yml_file = nfile
-
-    with open(app_yml_file, "w") as f:
-        f.write(app_yml)
-
-    # Template appinsts config
-    appinsts_tmpl_file = os.path.join(os.path.dirname(__file__), "appinsts.yml.tmpl")
-    with open(appinsts_tmpl_file) as f:
-        appinsts_tmpl = f.read()
-
-    t = Template(appinsts_tmpl)
-    appinsts_yml = t.substitute({
+        "deployenv": mc.deploy_env,
+        "flavor": None,
+        "ports": None,
+        "cluster": None,
         "cloudlet": mc.cloudlet,
         "cloudletorg": mc.cloudlet_org,
-        "devorg": org,
-    })
+    }
 
-    appinsts_yml_file = os.path.join(repo_conf_dir, "appinsts.yml")
-    if os.path.exists(appinsts_yml_file):
-        nfile = appinsts_yml_file + ".new"
-        print("File exists: {0}. Writing to {1}".format(
-            appinsts_yml_file, nfile))
-        appinsts_yml_file = nfile
+    workflow, data = github.load_workflow(data)
+    app, data = github.load_app(data)
+    appinst, data = github.load_appinst(data)
 
-    with open(appinsts_yml_file, "w") as f:
-        f.write(appinsts_yml)
+    flavors = mc.flavors.keys()
+    data["flavor"] = prompt("Pick a flavor", choices=flavors, default=data["flavor"])
+    data["ports"] = prompt("App ports [Eg: \"tcp:5678,tcp:9091,udp:9091\"]", default=data["ports"])
+    if not data["cluster"]:
+        # Compute a cluster name
+        data["cluster"] = "{}-cluster".format(app_name)
+    data["cluster"] = prompt("Cluster name", default=data["cluster"])
 
-    # Template github workflow config
-    workflow_tmpl_file = os.path.join(os.path.dirname(__file__), "workflow.tmpl")
-    with open(workflow_tmpl_file) as f:
-        workflow_tmpl = f.read()
-
-    t = Template(workflow_tmpl)
-    workflow = t.substitute({
-        "appname": app_name,
-        "deployenv": mc.deploy_env,
-        "devorg_lc": org.lower(),
-        "docker": docker,
-    })
-
-    workflow_file = os.path.join(repo_github_dir, "main.yml")
-    if os.path.exists(workflow_file):
-        nfile = workflow_file + ".new"
-        print("File exists: {0}. Writing to {1}".format(
-            workflow_file, nfile))
-        workflow_file = nfile
-
-    with open(workflow_file, "w") as f:
-        f.write(workflow)
+    github.dump_workflow(workflow, data)
+    github.dump_app(app, data)
+    github.dump_appinst(appinst, data)
 
     print("""
 Modify the following files as per your requirements:
@@ -462,13 +505,13 @@ Modify the following files as per your requirements:
 - .mobiledgex/app.yml
 - .mobiledgex/appinsts.yml
 
-Add your console credentials as Github secrets:
+Make sure you have added your credentials for "{}" as Github secrets:
 - MOBILEDGEX_USERNAME
 - MOBILEDGEX_PASSWORD
 See: https://docs.github.com/en/actions/reference/encrypted-secrets
 
 Tag a release of your app.
-""")
+""".format(mc.host))
 
 def edgebox_version(args):
     print("{} ({})".format(edgebox.__version__, RELEASE))
@@ -513,6 +556,9 @@ def main():
     parser_version.set_defaults(func=edgebox_version)
 
     args = parser.parse_args()
+    if args.debug:
+        logging.getLogger().setLevel(logging.DEBUG)
+
     args.func(args)
 
 if __name__ == "__main__":
