@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
 from datetime import datetime
+import base64
 import getpass
 import json
 import logging
@@ -13,14 +14,18 @@ import sys
 
 RESERVED_CLOUDLET_ORGS = ( "edgebox", "mobiledgex" )
 DEF_CONSOLE = "console.mobiledgex.net"
+DOCKER_CONFIG = os.path.expanduser("~/.docker/config.json")
 
 def prompt(text, default=None, choices=None, validate=None):
     prompt_str = text
     if choices:
         choices = [str(x) for x in choices]
         prompt_str += " (one of: " + ", ".join(choices) + ")"
-    if default:
+    if default and (not choices or default in choices):
         prompt_str += " (\"{0}\")".format(default)
+    else:
+        # Invalid default; ignore
+        default = None
     prompt_str += ": "
 
     reply = None
@@ -70,12 +75,14 @@ class MC(dict):
         self._token = None
         self._regions = None
         self._orgs = None
+        self._flavors = None
         self._roles = None
         self._location_defaults = None
         self._location_name = None
         self.revalidate = False
         self.artf_username = None
         self.artf_password = None
+        self._docker_logged_in = None
 
         if os.path.exists(varsfile):
             with open(varsfile, "r") as f:
@@ -109,6 +116,60 @@ class MC(dict):
 
         return self.params[key]
 
+    @property
+    def docker(self):
+        if self.deploy_env == "main":
+            return "docker.mobiledgex.net"
+        return "docker-{}.mobiledgex.net".format(self.deploy_env)
+
+    def _get_docker_auths(self):
+        try:
+            with open(DOCKER_CONFIG) as f:
+                config = json.load(f)
+            auths = config["auths"]
+        except Exception as e:
+            logging.debug("Failed to load docker config: {}".format(e))
+            auths = {}
+        return auths
+
+    def _docker_logged_in_macos(self):
+        auths = self._get_docker_auths()
+        if self.docker in auths:
+            cmd = ["security", "find-internet-password", "-a", self.username,
+                   "-s", self.docker]
+            logging.debug("Checking if docker creds are in macOS keychain: {}".format(
+                " ".join(cmd)))
+            p = subprocess.Popen(cmd,
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE,
+                                 universal_newlines=True)
+            out, err = p.communicate()
+            logging.debug("Out: {}, Err: {}".format(out, err))
+            return p.returncode == 0
+        return False
+
+    @property
+    def docker_logged_in(self):
+        if self._docker_logged_in is None:
+            if 'darwin' in sys.platform:
+                self._docker_logged_in = self._docker_logged_in_macos()
+            else:
+                # Assume that the user account is stored in the config file
+                self._docker_logged_in = False
+                auth = self._get_docker_auths().get(self.docker)
+                if auth:
+                    try:
+                        auth_decoded = base64.b64decode(auth["auth"]).decode('ascii')
+                        username = auth_decoded.split(":")[0]
+                        if username == self.username:
+                            self._docker_logged_in = True
+                        else:
+                            logging.debug("Docker logged in as user: {}".format(username))
+                    except Exception as e:
+                        logging.debug("Failed to decode docker auth: {}",format(e))
+
+        return self._docker_logged_in
+
     @property
     def username(self):
         if not self._username:
@@ -116,7 +177,7 @@ class MC(dict):
             logging.debug("Looking up username in env: " + console_username_env)
             u = os.environ.get(console_username_env)
             if not u:
-                u = prompt("Console username", getpass.getuser())
+                u = prompt("Console username for {}".format(self.host), getpass.getuser())
             self._username = u
         return self._username
 
@@ -138,7 +199,8 @@ class MC(dict):
             if 'darwin' in sys.platform:
                 self._password = self._password_macos()
             if not self._password:
-                self._password = getpass.getpass(prompt="Console password: ")
+                self._password = getpass.getpass(prompt="Console password for {}: ".format(
+                    self.host))
         return self._password
 
     @property
@@ -163,7 +225,7 @@ class MC(dict):
 
         return self._token
 
-    def call(self, api, method="POST", token=None, data={}, **kwargs):
+    def call(self, api, method="POST", timeout=180, token=None, data={}, **kwargs):
         if not data:
             data = kwargs
         if not token:
@@ -175,7 +237,8 @@ class MC(dict):
         r = requests.request(method, "https://{0}/api/v1/auth/{1}".format(
                                         self.host, api),
                              headers=headers,
-                             json=data)
+                             json=data,
+                             timeout=timeout)
         logging.debug("Response: {0}".format(r.text))
         if r.status_code != requests.codes.ok:
             raise Exception("API call failed: {0}: {1} {2}".format(
@@ -217,6 +280,17 @@ class MC(dict):
                 self._orgs[org["Name"]] = org["Type"]
         return self._orgs
 
+    @property
+    def flavors(self):
+        if not self._flavors:
+            self._flavors = {}
+            for flavor in self.call("ctrl/ShowFlavor",
+                                    data={
+                                        "region": self.region,
+                                    }):
+                self._flavors[flavor["key"]["name"]] = flavor
+        return self._flavors
+
     @property
     def roles(self):
         if not self._roles:
@@ -305,6 +379,11 @@ class MC(dict):
             self.params[key] = prompt("Cloudlet", default=default)
         return self.params[key]
 
+    @property
+    def cloudlet_pool(self):
+        return "-".join(("edgebox", self.cloudlet_org.lower(),
+                         re.sub(r"[^\d\w]", "-", self.username), "pool"))
+
     def validate_org(self, org):
         if org.lower() in RESERVED_CLOUDLET_ORGS:
             print("{0} is a reserved org. Please pick another.".format(org))
@@ -503,6 +582,86 @@ class MC(dict):
                                  },
                              },
                              "region": self.region,
+                         }, timeout=10)
+
+    def get_cloudlet_pool(self):
+        return self.call("ctrl/ShowCloudletPool",
+                         data={
+                             "region": self.region,
+                             "cloudletpool": {
+                                 "key": {
+                                     "name": self.cloudlet_pool,
+                                     "organization": self.cloudlet_org,
+                                 },
+                             },
+                         })
+
+    def create_cloudlet_pool(self):
+        return self.call("ctrl/CreateCloudletPool",
+                         data={
+                             "region": self.region,
+                             "cloudletpool": {
+                                 "key": {
+                                     "name": self.cloudlet_pool,
+                                     "organization": self.cloudlet_org,
+                                 },
+                             },
+                         })
+
+    def add_cloudlet_to_pool(self):
+        return self.call("ctrl/AddCloudletPoolMember",
+                         data={
+                             "region": self.region,
+                             "cloudletpoolmember": {
+                                 "key": {
+                                     "name": self.cloudlet_pool,
+                                     "organization": self.cloudlet_org,
+                                 },
+                                 "cloudlet_name": self.cloudlet,
+                             },
+                         })
+
+    def link_org_to_pool(self, org):
+        return self.call("orgcloudletpool/create",
+                         data={
+                             "cloudletpool": self.cloudlet_pool,
+                             "cloudletpoolorg": self.cloudlet_org,
+                             "region": self.region,
+                             "org": org,
+                         })
+
+    def unlink_org_from_pool(self,org):
+        return self.call("orgcloudletpool/delete",
+                         data={
+                             "cloudletpool": self.cloudlet_pool,
+                             "cloudletpoolorg": self.cloudlet_org,
+                             "region": self.region,
+                             "org": org,
+                         })
+
+    def show_cloudlet_pool_orgs(self, org=None):
+        data = self.call("orgcloudletpool/show",
+                         data={
+                             "cloudletpool": self.cloudlet_pool,
+                             "cloudletpoolorg": self.cloudlet_org,
+                             "region": self.region,
+                         })
+        # Explicitly filter orgs for pool
+        return [ x for x in data if x["Region"] == self.region \
+                                and x["CloudletPoolOrg"] == self.cloudlet_org \
+                                and x["CloudletPool"] == self.cloudlet_pool \
+                                and (not org or x["Org"] == org) ]
+
+    def delete_cloudlet_pool(self):
+        return self.call("ctrl/DeleteCloudletPool",
+                         data={
+                             "region": self.region,
+                             "cloudletpool": {
+                                 "key": {
+                                     "name": self.cloudlet_pool,
+                                     "organization": self.cloudlet_org,
+                                 },
+                             },
                          })
 
     def validate(self):
@@ -549,7 +708,8 @@ class MC(dict):
     Controller: {controller}
     Cloudlet Org: {cloudlet-org}
     Cloudlet: {cloudlet}
+    Cloudlet Pool: {cloudlet_pool}
     Latitude: {latitude}
     Longitude: {longitude}
     Output Dir: {outputdir}
-""".format(**self.params, outputdir=self.logdir)
+""".format(**self.params, cloudlet_pool=self.cloudlet_pool, outputdir=self.logdir)
